#!/usr/local/perl/perl -w
#
# AAFID project, COAST Laboratory, 1998.
#
# Susana Soriano, August 11, 1998.
#
# $Id: Fproc.pm,v 1.3 1998/09/03 21:15:47 zamboni Exp $
#

package Filter::Fproc;

use AAFID::Filter;
use AAFID::Entity;
use AAFID::Common;
use vars qw(
	%PARAMETERS
	@ISA
	$ps
	    );

  sub makefield;
  sub makeline;
  %PARAMETERS=( SOCKET_NAME=> '/tmp/catsock',
#		LOGFILE_NAME=> '/var/log/inetdlog'
		CHECK_PERIOD=> 10
		);

@ISA=qw(AAFID::Filter);

# The following variables are specific of the logfile and name of
# the socket that is wanted.

=pod

This is a filter which does not use a logfile. Instead, it obtain information
about which process are running by the command "ps". This filter uses the 
basic subroutines that are defined in Filter.pm, and this filter just defines 
the specific subroutines that have to be used for itself.

=head1 Defining how can be obtained the log.

Init_log and getline are two subroutines that are defined by default
in the package Filter.pm if the log is extracted by a file; however, if
this is not the case, they have to be redefined by the user to extract the
information without any logfile.

Note: The subroutine getline must return an undef function in order to avoid
an infinite loop.
=cut

sub Init_log {
	$ps=IO::File->new("ps -fea |");
}

sub getline {
 my $self=checkref(shift);
 if (defined($ps)){
	if ($ps->eof){
		$self->Init_log;
		return undef;
	}
	else{
           return scalar (<$ps>);
        }
   }
    else {
           return undef;
        }
}

=head1 Getting the patterns and Selecting the information.

The agent sends patterns to the filter in order to select specific information.
The filter split the lines into fields to be more careful about what information
is choosen. In this way, the filter looks for the line on the logfile which
matches with the pattern and reconstructs it to the original format. Once
this process is done, the filter send the information in the original format
to the agent.
=cut

	
sub makefield {
        my $self=checkref(shift);
        my $line=shift;
        my @subline;
        my @substring;
	my @subpid;
	my @pid;
        @subline=split(/\s+/,$line);
	if ($subline[6] =~ /[^0-9]/) {
	    $subline[9] = $subline[8];
	    $subline[8] = $subline[7];
	    $subline[7] = $subline[6];
	    $subline[6] = '-';
	}
	
	if ($subline[7] =~ /defunct/) {
	   $subline[8] = "-";
           $subline[9] = "-";
	}		
        return {Uid	=> $subline[1],
                Pid	=> $subline[2],
                Ppid	=> $subline[3],
                C	=> $subline[4],
                Month	=> $subline[5],
		Day	=> $subline[6],
                Tty	=> $subline[7],
                Time	=> $subline[8],
                Cmd	=> $subline[9],
                }
}

sub makeline{
	my $self=checkref(shift);
        my %line=@_;
        return "$line{Uid} $line{Pid} $line{Ppid} $line{C} $line{Month} $line{Day} $line{Tty} $line{Time} $line{Cmd}";
        }

_EndOfEntity;

#
# $Log: Fproc.pm,v $
# Revision 1.3  1998/09/03 21:15:47  zamboni
# Made some fixes (change Fproc to Filter::Fproc, etc.)
#
# Revision 1.2  1998/09/03 04:15:23  zamboni
# Removed the mail header lines.
#
#
